/*
 * This file is part of the qsl package.
 * 
 * qsl is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * qsl is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * Copyright 2011 Bilge Karacali 
 *
 * If you use the algorithm, please cite:
 * 
 * B. Karacali, "Quasi-supervised learning for biomedical data analysis,"
 * Pattern Recognition, Vol. 43, No. 10, p. 3674-3682, 2010
 * 
 */

#include "mex.h"
#include "math.h"

double qslmodule(double *ys, int ell, double n)
{
    double cell0, cell1;
    int i, k, n0, n1, kstar;
    double f1, nf1;
    double *Pin;
    
    //printf("ell = %d, n = %d\n",ell,(int)n);
    Pin=(double *)malloc(sizeof(double)*ell);
    
    n1=0;
    for (i=0;i<ell;i++)
    {
        //printf(".");
        n1+=(int)(ys[i]);
    }
    n0=ell-n1;
    
	cell0=n0;
    cell1=n1;
    //printf("cell0 = %d, cell1 = %d\n",(int)cell0,(int)cell1);
    
    kstar=-1;
    for (i=0;(i<ell-1)&&(kstar<0);i++)
    {
        if (ys[i])
        {
            Pin[i]=n/cell1;
            cell1=cell1-1;
        } else {
            Pin[i]=n/cell0;
            cell0=cell0-1;
        }
        if ((cell0<n)||(cell1<n))
            kstar=i;
        //printf("Pin[%d] = %.4f\n",i,Pin[i]);
        //printf("cell0 = %d, cell1 = %d\n",(int)cell0,(int)cell1);
    }
    //printf("kstar = %d\n",kstar);
    
    f1=(double)(ys[kstar]);
    //printf("f1 = %.4f\n",f1);
    for (k=kstar-1;k>=0;k--)
    {
        nf1=Pin[k]*((double)ys[k])+(1.0-Pin[k])*f1;
        f1=nf1;
        //printf("f1 = %.4f\n",f1);
    }
    
    free(Pin);
    
    return f1;
}


void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    double *ys, *nptr, *f1ptr;
    double n, ell;
    const int *dims;
    
    ys = mxGetPr(prhs[0]);
    dims = mxGetDimensions(prhs[0]);
    ell = dims[0]*dims[1];
    
    
    nptr = mxGetPr(prhs[1]);
    n=nptr[0];
    
    plhs[0] = mxCreateDoubleScalar(0.0);
    f1ptr = mxGetPr(plhs[0]);
    f1ptr[0] = qslmodule(ys,ell,n);
    
    return;
}
    
    


